import 'package:knt_bloc/knt_bloc.dart';
import 'package:native_admob_flutter/native_admob_flutter.dart';

import '../repository/monetized_repo.dart';
import '../service/admob_service.dart';

/// Handles generating & loading [BannerAdController]/[NativeAdController].
///
/// These controller will be used to create [BannerAd]/[NativeAd] widget.
class StaticAdsBloc extends BaseBloc {
  final AdmobService _admobService;
  final MonetizedRepo _monetizedRepo;

  BannerAdController? _currentBannerAdController;
  late int _retryLoadBannerAd;

  StaticAdsBloc(this._admobService, this._monetizedRepo) {
    _resetData();
  }

  @override
  Stream<BaseState> handleEvent(BaseEvent event) async* {
    yield* super.handleEvent(event);
    switch (event.runtimeType) {
      case PrepareStaticAdEvent:
        yield* _handlePrepareStaticAdEvent(event as PrepareStaticAdEvent);
        break;

      case RefreshAdEvent:
        yield* _handleRefreshAdEvent(event as RefreshAdEvent);
        break;

      case _LoadDoneStaticAdEvent:
        yield* _handleLoadDoneStaticAdEvent(event as _LoadDoneStaticAdEvent);
        break;
    }
  }

  Stream<BaseState> _handlePrepareStaticAdEvent(
      PrepareStaticAdEvent event) async* {
    await _createAndLoadBannerAd();
  }

  Stream<BaseState> _handleRefreshAdEvent(RefreshAdEvent event) async* {
    yield* _handleRefreshAd();
  }

  Stream<BaseState> _handleLoadDoneStaticAdEvent(
      _LoadDoneStaticAdEvent event) async* {
    yield* _handleRefreshAd();
  }

  Stream<BaseState> _handleRefreshAd() async* {
    final purchasedIap = await _monetizedRepo.fetchPurchasedIapStatus();
    if (purchasedIap) {
      _resetData();
    }
    yield LoadedStaticAdState(
      bannerAdController: _currentBannerAdController,
    );
  }

  void _resetData() {
    _currentBannerAdController = null;
    _retryLoadBannerAd = 0;
  }

  Future<void> _createAndLoadBannerAd({bool forceLoadAndShow = true}) async {
    _currentBannerAdController?.dispose();
    final onLoaded = () {
      _retryLoadBannerAd = 0;
      if (forceLoadAndShow) {
        add(_LoadDoneStaticAdEvent());
      }
    };
    final onLoadFailed = () async {
      if (_retryLoadBannerAd < _admobService.maxRetryReloadAds) {
        _retryLoadBannerAd++;
        await _createAndLoadBannerAd();
      }
    };
    _currentBannerAdController = _admobService.createBannerAdController(
      onLoaded: onLoaded,
      onLoadFailed: onLoadFailed,
    );
    await _currentBannerAdController?.load();
  }
}

// region EVENT
class PrepareStaticAdEvent extends BaseEvent {}

class RefreshAdEvent extends BaseEvent {}

class _LoadDoneStaticAdEvent extends BaseEvent {}
// endregion

// region STATE
class LoadedStaticAdState extends BaseState {
  final BannerAdController? bannerAdController;

  const LoadedStaticAdState({this.bannerAdController});
}
// endregion
